/* DO NOT EDIT! This file has been generated by gen-cta-vic.py from ANSI-CTA-861-I Errata FINAL.pdf. */

#include <libdisplay-info/cta-vic.h>

const struct di_cta_vic_video_format _di_cta_vic_video_formats[] = {
	[60] = {
		.vic = { .code = 60 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 59400000,
		.h_front = 1760,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[65] = {
		.vic = { .code = 65 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 59400000,
		.h_front = 1760,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[61] = {
		.vic = { .code = 61 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 2420,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[66] = {
		.vic = { .code = 66 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 2420,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[62] = {
		.vic = { .code = 62 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 1760,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[67] = {
		.vic = { .code = 67 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 1760,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[108] = {
		.vic = { .code = 108 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 90000000,
		.h_front = 960,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[109] = {
		.vic = { .code = 109 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 90000000,
		.h_front = 960,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[79] = {
		.vic = { .code = 79 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 59400000,
		.h_front = 1360,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[80] = {
		.vic = { .code = 80 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 59400000,
		.h_front = 1228,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[81] = {
		.vic = { .code = 81 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 59400000,
		.h_front = 700,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[110] = {
		.vic = { .code = 110 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 99000000,
		.h_front = 810,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[32] = {
		.vic = { .code = 32 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 638,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[72] = {
		.vic = { .code = 72 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 638,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[33] = {
		.vic = { .code = 33 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[73] = {
		.vic = { .code = 73 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[34] = {
		.vic = { .code = 34 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[74] = {
		.vic = { .code = 74 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[111] = {
		.vic = { .code = 111 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 638,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[112] = {
		.vic = { .code = 112 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 638,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[86] = {
		.vic = { .code = 86 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 99000000,
		.h_front = 998,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 11,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[87] = {
		.vic = { .code = 87 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 90000000,
		.h_front = 448,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[88] = {
		.vic = { .code = 88 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 118800000,
		.h_front = 768,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[113] = {
		.vic = { .code = 113 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 198000000,
		.h_front = 998,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 11,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[93] = {
		.vic = { .code = 93 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 1276,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[103] = {
		.vic = { .code = 103 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 1276,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[94] = {
		.vic = { .code = 94 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 1056,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[104] = {
		.vic = { .code = 104 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 1056,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[95] = {
		.vic = { .code = 95 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 176,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[105] = {
		.vic = { .code = 105 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 176,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[114] = {
		.vic = { .code = 114 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 1276,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[116] = {
		.vic = { .code = 116 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 1276,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[98] = {
		.vic = { .code = 98 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 1020,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[99] = {
		.vic = { .code = 99 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 968,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[100] = {
		.vic = { .code = 100 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 88,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[115] = {
		.vic = { .code = 115 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 1020,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[121] = {
		.vic = { .code = 121 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 396000000,
		.h_front = 1996,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 22,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[122] = {
		.vic = { .code = 122 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 396000000,
		.h_front = 1696,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 22,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[123] = {
		.vic = { .code = 123 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 396000000,
		.h_front = 664,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 22,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[124] = {
		.vic = { .code = 124 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 742500000,
		.h_front = 746,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 297,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[194] = {
		.vic = { .code = 194 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 2552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[202] = {
		.vic = { .code = 202 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 2552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[195] = {
		.vic = { .code = 195 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 2352,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[203] = {
		.vic = { .code = 203 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 2352,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[196] = {
		.vic = { .code = 196 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[204] = {
		.vic = { .code = 204 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[197] = {
		.vic = { .code = 197 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2376000000,
		.h_front = 2552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[205] = {
		.vic = { .code = 205 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2376000000,
		.h_front = 2552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[210] = {
		.vic = { .code = 210 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1485000000,
		.h_front = 1492,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 594,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[211] = {
		.vic = { .code = 211 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1485000000,
		.h_front = 2492,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[212] = {
		.vic = { .code = 212 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 1485000000,
		.h_front = 288,
		.h_sync = 176,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[213] = {
		.vic = { .code = 213 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2970000000,
		.h_front = 1492,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 594,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[17] = {
		.vic = { .code = 17 },
		.h_active = 720,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 12,
		.h_sync = 64,
		.h_back = 68,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[18] = {
		.vic = { .code = 18 },
		.h_active = 720,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 12,
		.h_sync = 64,
		.h_back = 68,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[21] = {
		.vic = { .code = 21 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 27000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[22] = {
		.vic = { .code = 22 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 27000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[23] = {
		.vic = { .code = 23 },
		.h_active = 1440,
		.v_active = 288,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[24] = {
		.vic = { .code = 24 },
		.h_active = 1440,
		.v_active = 288,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[29] = {
		.vic = { .code = 29 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 24,
		.h_sync = 128,
		.h_back = 136,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[30] = {
		.vic = { .code = 30 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 24,
		.h_sync = 128,
		.h_back = 136,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[25] = {
		.vic = { .code = 25 },
		.h_active = 2880,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 48,
		.h_sync = 252,
		.h_back = 276,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[26] = {
		.vic = { .code = 26 },
		.h_active = 2880,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 48,
		.h_sync = 252,
		.h_back = 276,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[27] = {
		.vic = { .code = 27 },
		.h_active = 2880,
		.v_active = 288,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 48,
		.h_sync = 252,
		.h_back = 276,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[28] = {
		.vic = { .code = 28 },
		.h_active = 2880,
		.v_active = 288,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 48,
		.h_sync = 252,
		.h_back = 276,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[37] = {
		.vic = { .code = 37 },
		.h_active = 2880,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 48,
		.h_sync = 256,
		.h_back = 272,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[38] = {
		.vic = { .code = 38 },
		.h_active = 2880,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 48,
		.h_sync = 256,
		.h_back = 272,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[19] = {
		.vic = { .code = 19 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 440,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[68] = {
		.vic = { .code = 68 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 440,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[82] = {
		.vic = { .code = 82 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 82500000,
		.h_front = 260,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[39] = {
		.vic = { .code = 39 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = true,
		.pixel_clock_hz = 72000000,
		.h_front = 32,
		.h_sync = 168,
		.h_back = 184,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 23,
		.v_sync = 5,
		.v_back = 57,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[20] = {
		.vic = { .code = 20 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = true,
		.pixel_clock_hz = 74250000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 2,
		.v_sync = 5,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[31] = {
		.vic = { .code = 31 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[75] = {
		.vic = { .code = 75 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[89] = {
		.vic = { .code = 89 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 185625000,
		.h_front = 548,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[96] = {
		.vic = { .code = 96 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 1056,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[106] = {
		.vic = { .code = 106 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 1056,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[101] = {
		.vic = { .code = 101 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 968,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[125] = {
		.vic = { .code = 125 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 742500000,
		.h_front = 1096,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[198] = {
		.vic = { .code = 198 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2376000000,
		.h_front = 2352,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[206] = {
		.vic = { .code = 206 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2376000000,
		.h_front = 2352,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[214] = {
		.vic = { .code = 214 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2970000000,
		.h_front = 2492,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[1] = {
		.vic = { .code = 1 },
		.h_active = 640,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 25175000,
		.h_front = 16,
		.h_sync = 96,
		.h_back = 48,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 10,
		.v_sync = 2,
		.v_back = 33,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[2] = {
		.vic = { .code = 2 },
		.h_active = 720,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 16,
		.h_sync = 62,
		.h_back = 60,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[3] = {
		.vic = { .code = 3 },
		.h_active = 720,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 16,
		.h_sync = 62,
		.h_back = 60,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[6] = {
		.vic = { .code = 6 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 27000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[7] = {
		.vic = { .code = 7 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 27000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[8] = {
		.vic = { .code = 8 },
		.h_active = 1440,
		.v_active = 240,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[9] = {
		.vic = { .code = 9 },
		.h_active = 1440,
		.v_active = 240,
		.interlaced = false,
		.pixel_clock_hz = 27000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[14] = {
		.vic = { .code = 14 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 32,
		.h_sync = 124,
		.h_back = 120,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[15] = {
		.vic = { .code = 15 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 32,
		.h_sync = 124,
		.h_back = 120,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[10] = {
		.vic = { .code = 10 },
		.h_active = 2880,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 76,
		.h_sync = 248,
		.h_back = 228,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[11] = {
		.vic = { .code = 11 },
		.h_active = 2880,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 76,
		.h_sync = 248,
		.h_back = 228,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[12] = {
		.vic = { .code = 12 },
		.h_active = 2880,
		.v_active = 240,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 76,
		.h_sync = 248,
		.h_back = 228,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[13] = {
		.vic = { .code = 13 },
		.h_active = 2880,
		.v_active = 240,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 76,
		.h_sync = 248,
		.h_back = 228,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[35] = {
		.vic = { .code = 35 },
		.h_active = 2880,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 64,
		.h_sync = 248,
		.h_back = 240,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[36] = {
		.vic = { .code = 36 },
		.h_active = 2880,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 64,
		.h_sync = 248,
		.h_back = 240,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[4] = {
		.vic = { .code = 4 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 110,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[69] = {
		.vic = { .code = 69 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 74250000,
		.h_front = 110,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[83] = {
		.vic = { .code = 83 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 99000000,
		.h_front = 260,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[5] = {
		.vic = { .code = 5 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = true,
		.pixel_clock_hz = 74250000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 2,
		.v_sync = 5,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[16] = {
		.vic = { .code = 16 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[76] = {
		.vic = { .code = 76 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[90] = {
		.vic = { .code = 90 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 198000000,
		.h_front = 248,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 11,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[97] = {
		.vic = { .code = 97 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 176,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[107] = {
		.vic = { .code = 107 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 176,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[102] = {
		.vic = { .code = 102 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 594000000,
		.h_front = 88,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[126] = {
		.vic = { .code = 126 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 742500000,
		.h_front = 164,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[199] = {
		.vic = { .code = 199 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2376000000,
		.h_front = 552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[207] = {
		.vic = { .code = 207 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2376000000,
		.h_front = 552,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 44,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[215] = {
		.vic = { .code = 215 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 2970000000,
		.h_front = 288,
		.h_sync = 176,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[42] = {
		.vic = { .code = 42 },
		.h_active = 720,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 12,
		.h_sync = 64,
		.h_back = 68,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[43] = {
		.vic = { .code = 43 },
		.h_active = 720,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 12,
		.h_sync = 64,
		.h_back = 68,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[44] = {
		.vic = { .code = 44 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[45] = {
		.vic = { .code = 45 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[41] = {
		.vic = { .code = 41 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 440,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[70] = {
		.vic = { .code = 70 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 440,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[84] = {
		.vic = { .code = 84 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 165000000,
		.h_front = 60,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 95,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[40] = {
		.vic = { .code = 40 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = true,
		.pixel_clock_hz = 148500000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 2,
		.v_sync = 5,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[64] = {
		.vic = { .code = 64 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[77] = {
		.vic = { .code = 77 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 528,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[91] = {
		.vic = { .code = 91 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 371250000,
		.h_front = 218,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 161,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[117] = {
		.vic = { .code = 117 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 1056,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[119] = {
		.vic = { .code = 119 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 1056,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[218] = {
		.vic = { .code = 218 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 800,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[127] = {
		.vic = { .code = 127 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1485000000,
		.h_front = 1096,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[200] = {
		.vic = { .code = 200 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 4752000000,
		.h_front = 2112,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[208] = {
		.vic = { .code = 208 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 4752000000,
		.h_front = 2112,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[216] = {
		.vic = { .code = 216 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 5940000000,
		.h_front = 2192,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[48] = {
		.vic = { .code = 48 },
		.h_active = 720,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 16,
		.h_sync = 62,
		.h_back = 60,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[49] = {
		.vic = { .code = 49 },
		.h_active = 720,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 54000000,
		.h_front = 16,
		.h_sync = 62,
		.h_back = 60,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[50] = {
		.vic = { .code = 50 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[51] = {
		.vic = { .code = 51 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 54000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[47] = {
		.vic = { .code = 47 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 110,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[71] = {
		.vic = { .code = 71 },
		.h_active = 1280,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 148500000,
		.h_front = 110,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 20,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[85] = {
		.vic = { .code = 85 },
		.h_active = 1680,
		.v_active = 720,
		.interlaced = false,
		.pixel_clock_hz = 198000000,
		.h_front = 60,
		.h_sync = 40,
		.h_back = 220,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 95,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[46] = {
		.vic = { .code = 46 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = true,
		.pixel_clock_hz = 148500000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 2,
		.v_sync = 5,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[63] = {
		.vic = { .code = 63 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[78] = {
		.vic = { .code = 78 },
		.h_active = 1920,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 297000000,
		.h_front = 88,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 36,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[92] = {
		.vic = { .code = 92 },
		.h_active = 2560,
		.v_active = 1080,
		.interlaced = false,
		.pixel_clock_hz = 495000000,
		.h_front = 548,
		.h_sync = 44,
		.h_back = 148,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 4,
		.v_sync = 5,
		.v_back = 161,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[118] = {
		.vic = { .code = 118 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 176,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[120] = {
		.vic = { .code = 120 },
		.h_active = 3840,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 176,
		.h_sync = 88,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[219] = {
		.vic = { .code = 219 },
		.h_active = 4096,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1188000000,
		.h_front = 88,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_256_135,
	},
	[193] = {
		.vic = { .code = 193 },
		.h_active = 5120,
		.v_active = 2160,
		.interlaced = false,
		.pixel_clock_hz = 1485000000,
		.h_front = 164,
		.h_sync = 88,
		.h_back = 128,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 8,
		.v_sync = 10,
		.v_back = 72,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[201] = {
		.vic = { .code = 201 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 4752000000,
		.h_front = 352,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[209] = {
		.vic = { .code = 209 },
		.h_active = 7680,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 4752000000,
		.h_front = 352,
		.h_sync = 176,
		.h_back = 592,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[217] = {
		.vic = { .code = 217 },
		.h_active = 10240,
		.v_active = 4320,
		.interlaced = false,
		.pixel_clock_hz = 5940000000,
		.h_front = 288,
		.h_sync = 176,
		.h_back = 296,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.v_front = 16,
		.v_sync = 20,
		.v_back = 144,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_POSITIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_64_27,
	},
	[52] = {
		.vic = { .code = 52 },
		.h_active = 720,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 12,
		.h_sync = 64,
		.h_back = 68,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[53] = {
		.vic = { .code = 53 },
		.h_active = 720,
		.v_active = 576,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 12,
		.h_sync = 64,
		.h_back = 68,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 5,
		.v_sync = 5,
		.v_back = 39,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[54] = {
		.vic = { .code = 54 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 108000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[55] = {
		.vic = { .code = 55 },
		.h_active = 1440,
		.v_active = 576,
		.interlaced = true,
		.pixel_clock_hz = 108000000,
		.h_front = 24,
		.h_sync = 126,
		.h_back = 138,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 2,
		.v_sync = 3,
		.v_back = 19,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[56] = {
		.vic = { .code = 56 },
		.h_active = 720,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 16,
		.h_sync = 62,
		.h_back = 60,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[57] = {
		.vic = { .code = 57 },
		.h_active = 720,
		.v_active = 480,
		.interlaced = false,
		.pixel_clock_hz = 108000000,
		.h_front = 16,
		.h_sync = 62,
		.h_back = 60,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 9,
		.v_sync = 6,
		.v_back = 30,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
	[58] = {
		.vic = { .code = 58 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 108000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_4_3,
	},
	[59] = {
		.vic = { .code = 59 },
		.h_active = 1440,
		.v_active = 480,
		.interlaced = true,
		.pixel_clock_hz = 108000000,
		.h_front = 38,
		.h_sync = 124,
		.h_back = 114,
		.h_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.v_front = 4,
		.v_sync = 3,
		.v_back = 15,
		.v_sync_polarity = DI_CTA_VIC_VIDEO_FORMAT_SYNC_NEGATIVE,
		.picture_aspect_ratio = DI_CTA_VIC_VIDEO_FORMAT_PICTURE_ASPECT_RATIO_16_9,
	},
};

const size_t _di_cta_vic_video_formats_len = 220;
